@@ -5,7 +5,7 @@
  * Author: Giovanni Giacobbi <johnny@themnemonic.org>
  * Copyright (C) 2002  Giovanni Giacobbi
  *
- * $Id: netcat.h,v 1.23 2002/06/09 08:56:50 themnemonic Exp $
+ * $Id: netcat.h,v 1.25 2002/06/16 09:48:04 themnemonic Exp $
  */
 
 /***************************************************************************
@@ -37,6 +37,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/uio.h>		/* needed for reading/writing vectors */
+#include <sys/param.h>		/* defines MAXHOSTNAMELEN and other stuff */
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* inet_ntop(), inet_pton() */
 
@@ -71,6 +72,10 @@
    string notation. */
 #define NETCAT_ADDRSTRLEN INET_ADDRSTRLEN
 
+/* FIXME: I should search more about this portnames standards. Currently
+   i'll fix my own size for this */
+#define NETCAT_MAXPORTNAMELEN 64
+
 /* Find out whether we can use the RFC 2292 extensions on this machine
    (I've found out only linux supporting this feature so far) */
 #ifdef HAVE_PKTINFO
@@ -88,6 +93,8 @@
 #ifndef INADDR_NONE
 # define INADDR_NONE 0xffffffff
 #endif
+
+/* FIXME: shall we really change this define? probably not. */
 #ifdef MAXHOSTNAMELEN
 # undef MAXHOSTNAMELEN		/* might be too small on aix, so fix it */
 #endif
@@ -108,11 +115,16 @@
 # endif
 #endif
 
-typedef struct {
-  unsigned char *head;
-  unsigned char *pos;
-  int len;
-} nc_buffer_t;
+/* Netcat basic operating modes */
+
+typedef enum {
+  NETCAT_UNSPEC,
+  NETCAT_CONNECT,
+  NETCAT_LISTEN,
+  NETCAT_TUNNEL
+} nc_mode_t;
+
+/* Recognized protocols */
 
 typedef enum {
   NETCAT_PROTO_UNSPEC,
@@ -120,18 +132,40 @@ typedef enum {
   NETCAT_PROTO_UDP
 } nc_proto_t;
 
-typedef struct {
-  char name[MAXHOSTNAMELEN];		/* dns name */
-  char addrs[MAXINETADDRS][24];		/* ascii-format IP addresses */
-  struct in_addr iaddrs[MAXINETADDRS];	/* real addresses: in_addr.s_addr: ulong */
-} nc_host_t;
+/* used for queues buffering and data tracking purposes.  The `head' field is
+   a pointer to the begin of the buffer segment, while `pos' indicates the
+   actual position of the data stream.  If `head' is NULL, it means that there
+   is no dynamically-allocated data in this buffer, *BUT* it MAY still contain
+   some local data segment (for example allocated inside the stack).
+   `len' indicates the length of the buffer starting from `pos'. */
 
 typedef struct {
-  char name[64];
+  unsigned char *head;
+  unsigned char *pos;
+  int len;
+} nc_buffer_t;
+
+/* this is the standard netcat hosts record.  It contains an "authoritative"
+   `name' field, which may be empty, and a list of IP addresses in the network
+   notation and in the dotted string notation. */
+
+typedef struct {
+  char name[MAXHOSTNAMELEN];			/* dns name */
+  char addrs[MAXINETADDRS][NETCAT_ADDRSTRLEN];	/* ascii-format IP addresses */
+  struct in_addr iaddrs[MAXINETADDRS];		/* real addresses */
+} nc_host_t;
+
+/* standard netcat port record.  It contains the port `name', which may be
+   empty, and the port number both as number and as string. */
+
+typedef struct {
+  char name[NETCAT_MAXPORTNAMELEN];
   char ascnum[8];
   unsigned short num;
 } nc_port_t;
 
+/* This is a more complex struct that holds socket records. [...] */
+
 typedef struct {
   int fd, domain, timeout;
   nc_proto_t proto;
