@@ -5,7 +5,7 @@
  * Author: Giovanni Giacobbi <johnny@themnemonic.org>
  * Copyright (C) 2002  Giovanni Giacobbi
  *
- * $Id: proto.h,v 1.24 2002/06/05 12:34:54 themnemonic Exp $
+ * $Id: proto.h,v 1.29 2002/06/16 14:13:59 themnemonic Exp $
  */
 
 /***************************************************************************
@@ -37,16 +37,17 @@ int netcat_flag_count(void);
 unsigned short netcat_flag_rand(void);
 
 /* misc.c */
-char *netcat_string_split(char **buf);
 int netcat_fhexdump(FILE *stream, char c, const void *data, size_t datalen);
+int netcat_snprintnum(char *str, size_t size, unsigned long number);
 void ncprint(int type, const char *fmt, ...);
+char *netcat_string_split(char **buf);
 void netcat_commandline_read(int *argc, char ***argv);
 void netcat_printhelp(char *argv0);
 void netcat_printversion(void);
 
 /* netcat.c */
-extern bool opt_listen, opt_tunnel, opt_numeric, opt_random, opt_hexdump,
-		opt_telnet, opt_zero;
+extern nc_mode_t netcat_mode;
+extern bool opt_numeric, opt_random, opt_hexdump, opt_telnet, opt_zero;
 extern int opt_interval, opt_verbose, opt_wait;
 extern char *opt_outputfile;
 extern nc_proto_t opt_proto;
@@ -54,9 +55,10 @@ extern FILE *output_fd;
 extern bool use_stdin;
 
 /* network.c */
-bool netcat_resolvehost(nc_host_t *dst, char *name);
+bool netcat_resolvehost(nc_host_t *dst, const char *name);
 bool netcat_getport(nc_port_t *dst, const char *port_string,
 		    unsigned short port_num);
+const char *netcat_strid(const nc_host_t *host, const nc_port_t *port);
 int netcat_inet_pton(const char *src, void *dst);
 const char *netcat_inet_ntop(const void *src);
 int netcat_socket_new(int domain, int type);
